package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import java.util.jar.JarOutputStream

 /**
 * Ear Delegate
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: A  Delegate for the creation of project EAR files
 * 
 * <p></p>
 */
/* 
 * @ant-target doEar
 */
class EarDelegate extends AbstractPluginTargetDelegate {
	
	private String ps = File.pathSeparator
	
	/*
	 * Scope of Libraries to package with EAR
	 * @ant-property-name ear.enterprise.scopes
	 */
	protected String enterpriseScopes
	/*
	 * Map to EarContent folder location of the EnterpriseApplication project
	 * @ant-property-name ear.enterprise.folder
	 */
	protected String enterpriseFolder
	/*
	 * Value to determine if compression is needed while archiving. 
	 * @ant-property-name ear.compress
	 */
	protected String compress
	/*
	 * For entries coming from existing archives, keep compression as it has been originally instead of compress attribute. 
	 * @ant-property-name ear.keep.compression
	 */
	protected String keepCompression
	/*
	 * Character encoding to use for filenames inside the archive. 
	 * @ant-property-name ear.encoding
	 */
	protected String encoding
	/*
	 * Indicates to update or override destination file if exists already. 
	 * @ant-property-name ear.update
	 */
	protected String update
	/*
	 * Behavior when a duplicate file is found. 
	 * @ant-property-name ear.duplicate
	 */
	protected String duplicate
	/*
	 * Whether file modification time will be rounded up to next even number of seconds. 
	 * @ant-property-name ear.roundup
	 */
	protected String roundup
	/*
	 * Non default level at which file compression should be performed.
	 * @ant-property-name ear.level
	 */
	protected String level
	/*
	 * Manifest file to use
	 * @ant-property-name ear.manifest
	 */
	protected String manifest
	/*
	 * Destination folder where the EAR archive to be placed
	 * @ant-property-name ear.destination.folder
	 */
	protected String destinationFolder
	/*
	 * Behaviour when no files match.
	 * @ant-property-name ear.when.manifest.only
	 */
	protected String whenManifestOnly
	/*
	 * Behavior when a manifest is found in zipfileset or zipgroupfileset.  
	 * @ant-property-name ear.fileset.manifest
	 */
	protected String filesetManifest
	/*
	 * Encoding used to read jar manifest. when not specified, defaults to platform encoding 
	 * @ant-property-name ear.manifest.encoding
	 */
	protected String manifestEncoding
	/*
	 * Folder where the runtime libraries are copied in to 
	 * @ant-property-name ear.libraries.folder
	 */
	protected String librariesFolder
	/*
	 * Archive type.
	 * @ant-property-name ear.type
	 */
	protected String type
	/*
	 * Boolean value to determine if the library version to be stripped from the library name.
	 * @ant-property-name ear.strip.version
	 */
	protected String stripVersion
	/*
	 * Enterprise File archive Name (without the extension)
	 * @ant-property-name ear.enterprise.file
	 */
	protected String enterpriseFile
	/*
	 * Application xml(deployment descriptor) file location
	 * @ant-property-name ear.enterprise.appxml
	 */
	protected String enterpriseAppxml
	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 EarDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	} 
	/**
	 * Setter method for member variable enterpriseScopes
	 * @param enterpriseScopes	
	 */	
	void setEnterpriseScopes( String enterpriseScopes ){
		this.enterpriseScopes = enterpriseScopes
	}
	/**
	 * Setter method for member variable enterpriseFolder
	 * @param enterpriseFolder	
	 */	
	void setEnterpriseFolder( String enterpriseFolder ){
		this.enterpriseFolder = enterpriseFolder
		if( !( enterpriseFolder.endsWith( '/' ) || ( enterpriseFolder.endsWith( '\\' ) ) ) )
			this.enterpriseFolder = this.enterpriseFolder + '/'
	}
	/**
	 * Setter method for member variable compress
	 * @param compress	
	 */	
	void setCompress( String compress ){
		this.compress = compress
	}
	/**
	 * Setter method for member variable keepCompression
	 * @param keepCompression	
	 */	
	void setKeepCompression( String keepCompression ){
		this.keepCompression = keepCompression
	}
	/**
	 * Setter method for member variable encoding
	 * @param encoding	
	 */	
	void setEncoding( String encoding ){
		this.encoding = encoding
	}
	/**
	 * Setter method for member variable update
	 * @param update	
	 */	
	void setUpdate( String update ){
		this.update = update
	}
	/**
	 * Setter method for member variable duplicate
	 * @param duplicate	
	 */	
	void setDuplicate( String duplicate ){
		this.duplicate = duplicate
	}
	/**
	 * Setter method for member variable roundup
	 * @param roundup	
	 */	
	void setRoundup( String roundup ){
		this.roundup = roundup
	}
	/**
	 * Setter method for member variable level
	 * @param level	
	 */	
	void setLevel( String level ){
		this.level = level
	}
	/**
	 * Setter method for member variable manifest
	 * @param manifest	
	 */	
	void setManifest( String manifest ){
		this.manifest = manifest
	}
	/**
	 * Setter method for member variable whenManifestOnly
	 * @param whenManifestOnly	
	 */	
	void setWhenManifestOnly( String whenManifestOnly ){
		this.whenManifestOnly = whenManifestOnly
	}
	/**
	 * Setter method for member variable filesetManifest
	 * @param filesetManifest	
	 */	
	void setFilesetManifest( String filesetManifest ){
		this.filesetManifest = filesetManifest
	}
	/**
	 * Setter method for member variable manifestEncoding
	 * @param manifestEncoding	
	 */	
	void setManifestEncoding( String manifestEncoding ){
		this.manifestEncoding = manifestEncoding
	}
	/**
	 * Setter method for member variable destinationFolder
	 * @param destinationFolder	
	 */	
	void setDestinationFolder( String destinationFolder ){
		this.destinationFolder = destinationFolder
		if( !( destinationFolder.endsWith( '/' ) || ( destinationFolder.endsWith( '\\' ) ) ) )
			this.destinationFolder = this.destinationFolder + '/'
	}
	/**
	 * Setter method for member variable librariesFolder
	 * @param librariesFolder	
	 */	
	void setLibrariesFolder( String librariesFolder ){
		this.librariesFolder = librariesFolder
	}
	/**
	 * Setter method for member variable type
	 * @param type	
	 */	
	void setType( String type ){
		this.type = type
	}
	/**
	 * Setter method for member variable stripVersion
	 * @param stripVersion	
	 */	
	void setStripVersion( String stripVersion ){
		this.stripVersion = stripVersion
	}
	/**
	 * Setter method for member variable enterpriseFile
	 * @param enterpriseFile	
	 */	
	void setEnterpriseFile( String enterpriseFile ){
		this.enterpriseFile = enterpriseFile
	}
	/**
	 * Setter method for member variable enterpriseAppxml
	 * @param enterpriseAppxml	
	 */	
	void setEnterpriseAppxml( String enterpriseAppxml ){
		this.enterpriseAppxml = enterpriseAppxml
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert enterpriseScopes != null, "Error due to Null value of ear.enterprise.scopes"
		assert enterpriseFolder != null, "Error due to Null value of ear.enterprise.folder"
		assert compress != null, "Error due to Null value of ear.compress"
		assert (compress.equalsIgnoreCase("true") || compress.equalsIgnoreCase("false")), "ear.compress has to be a boolean value"
		
		assert keepCompression != null, "Error due to Null value of ear.keep.compression"
		assert (keepCompression.equalsIgnoreCase("true") || keepCompression.equalsIgnoreCase("false")), "ear.keep.compression has to be a boolean value"
		
		assert encoding != null, "Error due to Null value of ear.encoding"
		assert update != null, "Error due to Null value of ear.update"
		assert (update.equalsIgnoreCase("true") || update.equalsIgnoreCase("false")), "ear.update has to be a boolean value"
		
		assert duplicate != null, "Error due to Null value of ear.duplicate"
		assert roundup != null, "Error due to Null value of ear.roundup"
		assert (roundup.equalsIgnoreCase("true") || roundup.equalsIgnoreCase("false")), "ear.roundup has to be a boolean value"
		
		assert level != null, "Error due to Null value of ear.level"
		assert manifest != null, "Error due to Null value of ear.manifest"
		assert filesetManifest != null, "Error due to Null value of ear.fileset.manifest"
		assert whenManifestOnly != null, "Error due to Null value of ear.when.manifest.only"
		assert manifestEncoding != null, "Error due to Null value of ear.manifest.encoding"
		assert destinationFolder != null, "Error due to Null value of ear.destination.folder"
		assert librariesFolder != null, "Error due to Null value of ear.libraries.folder"
		assert type != null, "Error due to Null value of ear.type"
		assert buildContext.get( "project.properties" ) != null, "Buildcontext is null"
		assert stripVersion != null, "Error due to Null value of ear.strip.version"
		assert (stripVersion.equalsIgnoreCase("true") || stripVersion.equalsIgnoreCase("false")), "ear.strip.version has to be a boolean value"
		
		assert enterpriseFile != null
		assert enterpriseAppxml != null
		
		def destDir = new File (destinationFolder)
		assert (destDir.exists() && destDir.isDirectory()), "ear.destination.folder should be a valid directory"
		def entFolder = new File (enterpriseFolder)
		assert (entFolder.exists() && entFolder.isDirectory()), "ear.enterprise.folder should be a valid directory"
	}	
	
	/**
	 * Concrete implementation of the AbstractDelegate.execute() method
	 */
	void doExecute(){		
		/* Construct Enterprise file name with extension along with directory location */
		String earDest = destinationFolder + enterpriseFile + '.' + type
		
		/* deployment descriptor file location */
		String appXml = enterpriseFolder + enterpriseAppxml
		
		/* Extract artifact name to be used in scope iteration*/
		String artifactName = ant.project.getProperty("global.artifactid")
		
		/* populate attributes for ear ant task */
		def earOptions = [destfile:earDest, 
		                  compress:compress,
		                  keepcompression:keepCompression,
		                  encoding:encoding,
		                  update:update,
		                  duplicate:duplicate,
		                  roundup:roundup,
		                  filesetmanifest:filesetManifest,
		                  whenmanifestonly:whenManifestOnly
		                 ]
		// check if a manifest file has been included, if not we want a default manifest building for us by ant
		if ( !manifest.equals( "" ) )
			earOptions.put( "manifest", manifest )
		if( !manifestEncoding.equals(""))
			earOptions.put( "manifestencoding", manifestEncoding )
		if( !level.equals(""))
			earOptions.put( "level", level )
		/* if update is set to false, add the appxml attribute */
		if( update.equals("false")) 
			earOptions.put( "appxml", appXml )

		// Delete the folder before performing a fresh copy
		ant.delete(dir:librariesFolder)
		ant.mkdir(dir:librariesFolder)
		
		/* Loop through the provided ear scopes for libraries */
		log.info "Coping dependencies for scopes $enterpriseScopes"
		enterpriseScopes.split(',').each{ scope ->
			if(buildContext.isSet("${artifactName}.${scope}.classpath" )){
				String dependencies = buildContext.get( "${artifactName}.${scope}.classpath" )	
				log.debug "Dependencies to be copied are: $dependencies"
				dependencies.split( "${ps}" ).each{	dependency ->					
					def libString = dependency.toString()
					ant.copy( file:dependency, todir:librariesFolder) 
				}
			}
		}
		
		/* Rename runtime libraries so as to make sure the version is stripped off */
		if(stripVersion.equalsIgnoreCase("true"))
			renameFiles(librariesFolder)
		
		/* Execute the ear task with loaded attributes */
		ant.ear( earOptions ){				
				ant.fileset	( dir: enterpriseFolder )
				ant.fileset ( dir: librariesFolder )
		}
		
		ant.property( name : "global.artifact.location", value : earDest )
	}
	
	/**
	 * Method to remove version number from file name. 
	 * Libraries when downloaded from Maven repository have their version appended part of the file name. 
	 * This should be removed while packaging enterprise library as there will be mismatch of the library in the package and the one specified in the deployment descriptor.
	 * @param location Folder containing the files to be renamed
	 */
	def renameFiles(String location) {
		def dir = new File(location)
		if(dir.exists() && dir.isDirectory()) {
			dir.eachFile() { file ->
				if(!file.isDirectory()) {
					def name = file.getName()
					def newName, newExtension
					def requiredIndex = -1
					def lastIndex =  name.lastIndexOf('.')
					name.eachWithIndex() { aChar, index->
						if(aChar.equals('-')) {
							boolean isDigit = Character.isDigit(name.charAt(index+1))
							if(isDigit)
								requiredIndex = index
						}
					}
					if(requiredIndex != -1)
						newName = name.substring(0,requiredIndex)
					if(lastIndex != -1)
						newExtension = name.substring(lastIndex)
					if(newName!= null && newExtension!= null) {
						File newFile = new File(location+newName+newExtension)
						file.renameTo(newFile)
					}
				}
			}	
		}
	}
	
}
